package de.assono.ec12;

import lotus.domino.*;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.*;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.Element;
import com.itextpdf.text.html.simpleparser.*;
import com.lowagie.text.pdf.PdfCell;

public class PDFExportForHTML {
	///////////////////////////////////////
	// constants (final static)
	private final static String PDF_FILE_NAME = "HTMLContent.pdf";
	private final static int CONVERT_RT_TO_HTML = 2;
	
	///////////////////////////////////////
	// class attributes (static)

	///////////////////////////////////////
	// attributes
	private String htmlContent;
	private String filePath;
	
	///////////////////////////////////////
	// associations
	
	///////////////////////////////////////
	// constructor
	public PDFExportForHTML(Session session, lotus.domino.Document doc) {
		try {			
			if (doc.getItemValueString("$NoteHasNativeMIME")=="1"){
				this.getHTMLContentFromMIME(session, doc);				
			} else {
				lotus.domino.Document mimeDoc = doc.getParentDatabase().createDocument();
				doc.copyAllItems(mimeDoc, true);
				mimeDoc.convertToMIME(CONVERT_RT_TO_HTML);
				this.getHTMLContentFromMIME(session, mimeDoc);	
				mimeDoc.recycle();
			}
			
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	///////////////////////////////////////
	// main method
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			NotesThread.sinitThread();
			Session session = NotesFactory.createSession();
						
			Database db = session.getDatabase("","Projekte\\Konferenzen\\ec12JavaNotes.nsf");
			System.out.println(db.getTitle() + " (" + db.getServer() + "!!"
					+ db.getFilePath() + ")");

			//Notes:///C12579C300546220/3656924109A5F51FC12579CA003C190B/05F28D219C9DA342C12579CC0060140B
			Document doc = db.getDocumentByUNID("05F28D219C9DA342C12579CC0060140B");
			
			PDFExportForHTML pdf = new PDFExportForHTML(session, doc);
			pdf.createPDF(true);
			
			doc.recycle();
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			NotesThread.stermThread();
		}

	}
	///////////////////////////////////////
	// class methods

	///////////////////////////////////////
	// instance methods
	/**
	 * Creates the pdf document
	 * 
	 * @param doOpen
	 *            if <code>true</code> the pdf document is openend
	 */
	public void createPDF(boolean doOpen) {
		try {
			
			String desktopPath = System.getProperty("user.home") + "/Desktop/";
			filePath = desktopPath + PDF_FILE_NAME;
			
			// step 1: creation of a document-object
			com.itextpdf.text.Document pdfDocument = new com.itextpdf.text.Document();
			try {
				// step 2:
				// we create a writer that listens to the document
				// and directs a PDF-stream to a file
				PdfWriter.getInstance(pdfDocument, new FileOutputStream(filePath));

				// step 3: we open the document
				pdfDocument.open();

				float[] widths = {0.3f, 17.4f};
				PdfPTable table = new PdfPTable(widths);

				PdfPCell cell = new PdfPCell(new Paragraph("Der Inhalt des Dokumentes",
						new Font(Font.FontFamily.HELVETICA, 16, Font.BOLD)));
				cell.setColspan(2);
				cell.setBorder(PdfCell.NO_BORDER);
				table.addCell(cell);

				//add an empty cell				
				cell = new PdfPCell(new Phrase(new Chunk("")));
				cell.setBorder(PdfCell.NO_BORDER);
				table.addCell(cell);
				
				//add content
				cell = new PdfPCell();
				java.util.List objects = HTMLWorker.parseToList(new StringReader(this.htmlContent), null, null);
				Iterator it = objects.iterator();
				while (it.hasNext()){
					Element element = (Element) it.next();
					cell.addElement(element);
				}
				
				cell.setBorder(PdfPCell.NO_BORDER);
				table.addCell(cell);
				pdfDocument.add(table);
				
				

			} catch (DocumentException de) {
				System.err.println(de.getMessage());
			} catch (IOException ioe) {
				System.err.println(ioe.getMessage());
			}

			// step 5: we close the document
			pdfDocument.close();

			if (doOpen){
				this.openPDF();
			}
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Opens the pdf document
	 * 
	 * @return true, false
	 */
	public boolean openPDF() {
		try {
			String osName = System.getProperty("os.name");
			// FOR WINDOWS 95 AND 98 USE COMMAND.COM
			if (osName.equals("Windows 95") || osName.equals("Windows 98")) {
				Runtime.getRuntime().exec(
						"command.com /C start acrord32 \"" + filePath + "\"");
			}
			// FOR WINDOWS NT/XP/2000 USE CMD.EXE
			else {
				Runtime.getRuntime().exec(
						"cmd.exe /C start acrord32 \"" + filePath + "\"");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}// openPDF
	
	private void getHTMLContentFromMIME(Session session, lotus.domino.Document doc){
		try {
			boolean isConvertMIME = session.isConvertMIME();
			session.setConvertMIME(false);
			Item item = doc.getFirstItem("Body");			
			if (item != null){
				MIMEEntity mime = item.getMIMEEntity();
				if (mime!=null){
					//htmlContent = mime.getContentAsText().trim();
					if (mime.getContentType().equalsIgnoreCase("multipart")){
						MIMEEntity mimeChild = mime.getFirstChildEntity();
						int counter = 0;
						while (mimeChild!=null){
							if (mimeChild.getContentType().equalsIgnoreCase("text")){
								mime = mimeChild;
							}							
							mimeChild = mimeChild.getNextSibling();
							if (counter++>10){
								throw new RuntimeException("MIMEChild getNextSibling() terminiert nicht.");
							}
						}						
					}
					//System.out.println(mime.getCharset() + " / " + mime.getContentType());
					Stream stream = session.createStream();
					mime.getContentAsText(stream, true);
					stream.setPosition(0);				
					//htmlContent = stream.readText().trim().replaceAll("(?im)\n", "<br>");
					htmlContent = stream.readText().trim();
					//System.out.println(htmlContent);
				} 
			}
			session.setConvertMIME(isConvertMIME);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
