package de.assono;

import java.applet.AudioClip;

import lotus.domino.AppletBase;
import lotus.domino.Database;
import lotus.domino.NotesException;
import lotus.domino.NotesThread;
import lotus.domino.Session;
import lotus.domino.View;

public class WarntonApplet extends AppletBase {

	static final int WAIT_TIME = 3000; // 3000 ms = 3 s

	boolean monitorActive = true;
	AudioClip clip = null;
	Session session = null;
	Database db = null;
	View monitoredView = null;

	public WarntonApplet() {
		super();
		System.out.println("Konstruktor");
	}

	public void notesAppletInit() {
		super.notesAppletInit();
		System.out.println("notesAppletInit()");

		clip = getAudioClip(getCodeBase(), "extreme_alarm.wav");

		try {
			NotesThread.sinitThread();
			session = getSession();
			System.out.println("Angemeldeter Benutzer: " + session.getCommonUserName());

			db = getContext(session).getDatabase();
			System.out.println("getContext(session).getDatabase(): " + (db == null ? "keine Datenbank?!" : db.getTitle()));

			if (db != null) {
				monitoredView = db.getView("MonitoringView");
				System.out.println("Monitoring View: " + (monitoredView == null ? "nicht gefunden?!" : monitoredView.getName()));
			}

		} catch (NotesException ne) {
			System.out.println("Notes-Fehler #" + ne.id + ": " + ne.text);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void notesAppletStart() {
		super.notesAppletStart();
		System.out.println("notesAppletStart()");

		monitorActive = true;
		monitorView();
	}

	void monitorView() {
		System.out.println("monitorView()");
		if (monitoredView != null) {
			try {
				do {
					monitoredView.refresh();
					int entriesInView = monitoredView.getEntryCount();
					System.out.println("getEntryCount: " + entriesInView);

					if (entriesInView > 0) {
						clip.play();
					} else if (entriesInView == 0) {
						clip.stop();
					}

					try { // warte ein paar Sekunden
						Thread.sleep(WAIT_TIME);
					} catch (InterruptedException ie) {
						// do nothing
					}
				} while (monitorActive);
			} catch (NotesException ne) {
				System.out.println("Notes-Fehler #" + ne.id + ": " + ne.text);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void notesAppletStop() {
		super.notesAppletStop();
		System.out.println("notesAppletStop()");

		clip.stop();
		monitorActive = false;
	}

	public void notesAppletDestroy() {
		super.notesAppletDestroy();
		System.out.println("notesAppletDestroy()");

		try {
			if (monitoredView != null) {
				monitoredView.recycle();
			}
			if (db != null) {
				db.recycle();
			}
			if (session != null) {
				closeSession(session);
			}
		} catch (NotesException ne) {
			System.out.println("Notes-Fehler #" + ne.id + ": " + ne.text);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			NotesThread.stermThread();
		}
	}

}
