package de.assono.ec12.xpages;

import lotus.domino.*;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.util.*;

import com.itextpdf.text.Chunk;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Font;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.Element;
import com.itextpdf.text.html.simpleparser.*;
import com.lowagie.text.pdf.PdfCell;

public class PDFExport {
///////////////////////////////////////
	// constants (final static)
	private final static int DEFAULT_IMAGE_COMPRESSION_LEVEL = 0;
	private final static int CONVERT_RT_TO_HTML = 2;
	
	public final static int OUTPUT_TO_FILE = 1;
	public final static int OUTPUT_TO_BYTEARRAY = 2;
	
	///////////////////////////////////////
	// class attributes (static)

	///////////////////////////////////////
	// attributes
	private String htmlContent;
	private String filePath;
	private String fileName;
	
	private com.itextpdf.text.Document pdfDocument;
	private ByteArrayOutputStream docStream;
	private PdfWriter writer; 
	
	///////////////////////////////////////
	// associations
	
	///////////////////////////////////////
	// constructor
	public PDFExport(Session session, lotus.domino.Document doc) {
		try {	
			this.fileName = doc.getItemValueString("Subject");
			String desktopPath = System.getProperty("user.home") + "/Desktop/";
			this.setFilePath(desktopPath);
			
			if (doc.getItemValueString("$NoteHasNativeMIME")=="1"){
				this.getHTMLContentFromMIME(session, doc);				
			} else {
				lotus.domino.Document mimeDoc = doc.getParentDatabase().createDocument();
				doc.copyAllItems(mimeDoc, true);
				mimeDoc.convertToMIME(CONVERT_RT_TO_HTML);
				this.getHTMLContentFromMIME(session, mimeDoc);	
				mimeDoc.recycle();	
			}
			
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	///////////////////////////////////////
	// main method
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			NotesThread.sinitThread();
			Session session = NotesFactory.createSession();
						
			Database db = session.getDatabase("","Projekte\\Konferenzen\\ec12JavaNotes.nsf");
			System.out.println(db.getTitle() + " (" + db.getServer() + "!!"
					+ db.getFilePath() + ")");

			//Notes:///C12579C300546220/3656924109A5F51FC12579CA003C190B/05F28D219C9DA342C12579CC0060140B
			Document doc = db.getDocumentByUNID("05F28D219C9DA342C12579CC0060140B");
			
			PDFExport pdf = new PDFExport(session, doc);
			//pdf.createPDF(PDFExport.OUTPUT_TO_FILE);
			//pdf.openPDFFile();
			
			pdf.createPDF(PDFExport.OUTPUT_TO_BYTEARRAY);
			FileOutputStream stream = new FileOutputStream(System.getProperty("user.home") + "/Desktop/test.pdf");
			pdf.saveToOutputStream(stream);
			
			doc.recycle();
			System.out.println("PDF generiert");
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			NotesThread.stermThread();
		}

	}
	///////////////////////////////////////
	// class methods

	///////////////////////////////////////
	// instance methods
	/**
	 * Creates the pdf document
	 * 
	 * @param outputType OUTPUT_TO_FILE or OUTPUT_TO_BYTEARRAY
	 */
	public void createPDF(int outputType) {
		try {
			
			
			
			// step 1: creation of a document-object
			this.pdfDocument = new com.itextpdf.text.Document();
			
			if (outputType == OUTPUT_TO_FILE){
				// step 2:
				// we create a writer that listens to the document
				// and directs a PDF-stream to a file
				this.writer = PdfWriter.getInstance(pdfDocument, 
				new FileOutputStream(this.getFilePathInclFilename()));
				
			} else if(outputType == OUTPUT_TO_BYTEARRAY){
				// step 2:
				// we create a writer that listens to the document
				// and directs a PDF-stream to a byte array output stream
				this.docStream = new ByteArrayOutputStream();
				this.writer = PdfWriter.getInstance(pdfDocument, docStream);
			} else {
				throw new IllegalArgumentException("Die Option " + outputType + " wird nicht untersttzt." );
			}
			
			this.writer.setCompressionLevel(DEFAULT_IMAGE_COMPRESSION_LEVEL);
			
			try {
				
				
				// step 3: we open the document
				pdfDocument.open();

				float[] widths = {0.3f, 17.4f};
				PdfPTable table = new PdfPTable(widths);

				PdfPCell cell = new PdfPCell(new Paragraph("Der Inhalt des Dokumentes",
						new Font(Font.FontFamily.HELVETICA, 16, Font.BOLD)));
				cell.setColspan(2);
				cell.setBorder(PdfCell.NO_BORDER);
				table.addCell(cell);

				//add an empty cell				
				cell = new PdfPCell(new Phrase(new Chunk("")));
				cell.setBorder(PdfCell.NO_BORDER);
				table.addCell(cell);
				
				//add content
				cell = new PdfPCell();
				java.util.List<Element> objects = HTMLWorker.parseToList(new StringReader(this.htmlContent), null, null);
				Iterator<Element> it = objects.iterator();
				while (it.hasNext()){
					Element element = it.next();
					cell.addElement(element);
				}
				
				cell.setBorder(PdfPCell.NO_BORDER);
				table.addCell(cell);
				pdfDocument.add(table);
				
				

			} catch (DocumentException de) {
				System.err.println(de.getMessage());
			} catch (IOException ioe) {
				System.err.println(ioe.getMessage());
			}

			// step 5: we close the document
			pdfDocument.close();

			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	

	private void getHTMLContentFromMIME(Session session, lotus.domino.Document doc){
		try {
			boolean isConvertMIME = session.isConvertMIME();
			session.setConvertMIME(false);
			Item item = doc.getFirstItem("Body");			
			if (item != null){
				MIMEEntity mime = item.getMIMEEntity();
				if (mime!=null){
					//htmlContent = mime.getContentAsText().trim();
					if (mime.getContentType().equalsIgnoreCase("multipart")){
						MIMEEntity mimeChild = mime.getFirstChildEntity();
						int counter = 0;
						while (mimeChild!=null){
							if (mimeChild.getContentType().equalsIgnoreCase("text")){
								mime = mimeChild;
							}							
							mimeChild = mimeChild.getNextSibling();
							if (counter++>10){
								throw new RuntimeException("MIMEChild getNextSibling() terminiert nicht.");
							}
						}						
					}
					//System.out.println(mime.getCharset() + " / " + mime.getContentType());
					Stream stream = session.createStream();
					mime.getContentAsText(stream, true);
					stream.setPosition(0);				
					//htmlContent = stream.readText().trim().replaceAll("(?im)\n", "<br>");
					htmlContent = stream.readText().trim();
					//System.out.println(htmlContent);
				} 
			}
			session.setConvertMIME(isConvertMIME);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Opens the pdf document with the Adobe Reader application
	 * 
	 * @return true, false
	 */
	public boolean openPDFFile() {
		try {
			String osName = System.getProperty("os.name");
			// FOR WINDOWS 95 AND 98 USE COMMAND.COM
			if (osName.equals("Windows 95") || osName.equals("Windows 98")) {
				Runtime.getRuntime().exec(
						"command.com /C start acrord32 \"" + this.getFilePathInclFilename() + "\"");
			}
			// FOR WINDOWS NT/XP/2000 USE CMD.EXE
			else {
				Runtime.getRuntime().exec(
						"cmd.exe /C start acrord32 \"" + this.getFilePathInclFilename() + "\"");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}// openPDF
	
	/**
	 * 
	 * @return The byte array output stream
	 */
	public ByteArrayOutputStream getDocumentStream() {
		return docStream;
	}
	
	/**
	 * Saves the byte array output stream to an output stream
	 * @param stream
	 * @throws IOException
	 */
	public void saveToOutputStream(java.io.OutputStream stream) throws IOException {
		try {
			if (this.pdfDocument.isOpen()) {
				this.pdfDocument.close();
			}
			stream.write(getDocumentStream().toByteArray());
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * 
	 * @return the file name without the file path
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * Sets the file name (excluding the file path)
	 * @param fileName
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	/**
	 * 
	 * @return The file path without the filename (ends with the platform specific path separator)
	 */
	public String getFilePath() {
		return filePath;
	}

	/**
	 * Sets the file path without the filename
	 * @param filePath
	 */
	public void setFilePath(String filePath) {
		String fileSeparator = File.pathSeparator;
		this.filePath = filePath;
		if (!this.filePath.endsWith(fileSeparator)){
			this.filePath += fileSeparator;			
		}
	}
	
	/**
	 * 
	 * @return The file path including the file name
	 */
	public String getFilePathInclFilename(){
		return this.getFilePath() + this.getFileName();		
	}
}
